@@ -41,31 +41,31 @@
 #include "Logger.h"
 #include "RecoverableException.h"
 #include "util.h"
-#include "Socket.h"
+#include "SocketCore.h"
 #include "SimpleRandomizer.h"
+#include "fmt.h"
 
 namespace aria2 {
 
-DHTConnectionImpl::DHTConnectionImpl(int family):
-  socket_(new SocketCore(SOCK_DGRAM)),
-  family_(family),
-  logger_(LogFactory::getInstance()) {}
+DHTConnectionImpl::DHTConnectionImpl(int family)
+  : socket_(new SocketCore(SOCK_DGRAM)),
+    family_(family)
+{}
 
 DHTConnectionImpl::~DHTConnectionImpl() {}
 
 bool DHTConnectionImpl::bind
-(uint16_t& port, const std::string& addr, IntSequence& ports)
+(uint16_t& port, const std::string& addr, SegList<int>& sgl)
 {
-  std::vector<int32_t> randPorts = ports.flush();
-  std::random_shuffle(randPorts.begin(), randPorts.end(),
+  std::vector<uint16_t> ports;
+  while(sgl.hasNext()) {
+    ports.push_back(sgl.next());
+  }
+  std::random_shuffle(ports.begin(), ports.end(),
                       *SimpleRandomizer::getInstance().get());
-  
-  for(std::vector<int32_t>::const_iterator portItr = randPorts.begin(),
-        eoi = randPorts.end(); portItr != eoi; ++portItr) {
-    if(!(0 < (*portItr) && (*portItr) <= 65535)) {
-      continue;
-    }
-    port = (*portItr);
+  for(std::vector<uint16_t>::const_iterator i = ports.begin(),
+        eoi = ports.end(); i != eoi; ++i) {
+    port = *i;
     if(bind(port, addr)) {
       return true;
     }
@@ -75,21 +75,17 @@ bool DHTConnectionImpl::bind
 
 bool DHTConnectionImpl::bind(uint16_t& port, const std::string& addr)
 {
-  int ipv = family_ == AF_INET?4:6;
+  const int ipv = (family_ == AF_INET) ? 4 : 6;
   try {
-    if(addr.empty()) {
-      socket_->bind(A2STR::NIL, port, family_);
-    } else {
-      socket_->bind(addr, port, family_);
-    }
+    socket_->bind(addr.c_str(), port, family_);
     socket_->setNonBlockingMode();
     std::pair<std::string, uint16_t> svaddr;
     socket_->getAddrInfo(svaddr);
     port = svaddr.second;
-    logger_->notice("IPv%d DHT: listening to port %d", ipv, port);
+    A2_LOG_NOTICE(fmt(_("IPv%d DHT: listening on UDP port %u"), ipv, port));
     return true;
   } catch(RecoverableException& e) {
-    logger_->error("Failed to bind for IPv%d DHT. port=%u", e, ipv, port);
+    A2_LOG_ERROR_EX(fmt("IPv%d DHT: failed to bind UDP port %u", ipv, port), e);
   }
   return false;
 }
