@@ -50,24 +50,40 @@ class SocketCore;
 class HttpHeader;
 class HttpHeaderProcessor;
 class DownloadEngine;
+class SocketRecvBuffer;
+class DiskWriter;
 
+enum RequestType {
+  RPC_TYPE_NONE,
+  RPC_TYPE_XML,
+  RPC_TYPE_JSON,
+  RPC_TYPE_JSONP
+};
+
+// HTTP server class handling RPC request from the client.  It is not
+// intended to be a generic HTTP server.
 class HttpServer {
 private:
   SharedHandle<SocketCore> socket_;
+  SharedHandle<SocketRecvBuffer> socketRecvBuffer_;
   SocketBuffer socketBuffer_;
-  DownloadEngine* e_;
   SharedHandle<HttpHeaderProcessor> headerProcessor_;
   SharedHandle<HttpHeader> lastRequestHeader_;
-  uint64_t lastContentLength_;
-  std::stringstream lastBody_;
+  int64_t lastContentLength_;
+  // How many bytes are consumed. The total number of bytes is
+  // lastContentLength_.
+  int64_t bodyConsumed_;
+  RequestType reqType_;
+  SharedHandle<DiskWriter> lastBody_;
   bool keepAlive_;
   bool gzip_;
   std::string username_;
   std::string password_;
-  bool acceptsPersistentConnection_;
   bool acceptsGZip_;
+  std::string allowOrigin_;
+  bool secure_;
 public:
-  HttpServer(const SharedHandle<SocketCore>& socket, DownloadEngine* e);
+  HttpServer(const SharedHandle<SocketCore>& socket);
 
   ~HttpServer();
 
@@ -75,16 +91,42 @@ public:
 
   bool receiveBody();
 
-  std::string getBody() const;
+  const std::string& getMethod() const;
 
   const std::string& getRequestPath() const;
 
-  void feedResponse(const std::string& text, const std::string& contentType);
+  int setupResponseRecv();
 
-  void feedResponse(const std::string& status,
-                    const std::string& headers,
-                    const std::string& text,
-                    const std::string& contentType);
+  std::string createPath() const;
+
+  std::string createQuery() const;
+
+  const SharedHandle<DiskWriter>& getBody() const
+  {
+    return lastBody_;
+  }
+
+  RequestType getRequestType() const
+  {
+    return reqType_;
+  }
+
+  void feedResponse(std::string& text, const std::string& contentType);
+
+  // Feeds HTTP response with the status code |status| (e.g.,
+  // 200). The |headers| is zero or more lines of HTTP header field
+  // and each line must end with "\r\n". The |text| is the response
+  // body. The |contentType" is the content-type of the response body.
+  void feedResponse(int status,
+                    const std::string& headers = "",
+                    const std::string& text = "",
+                    const std::string& contentType = "");
+
+  // Feeds "101 Switching Protocols" response. The |protocol| will
+  // appear in Upgrade header field. The |headers| is zero or more
+  // lines of HTTP header field and each line must end with "\r\n".
+  void feedUpgradeResponse(const std::string& protocol,
+                           const std::string& headers);
 
   bool authenticate();
 
@@ -95,10 +137,7 @@ public:
 
   bool sendBufferIsEmpty() const;
 
-  bool supportsPersistentConnection() const
-  {
-    return keepAlive_ && acceptsPersistentConnection_;
-  }
+  bool supportsPersistentConnection() const;
 
   bool supportsGZip() const
   {
@@ -113,7 +152,45 @@ public:
 
   void disableGZip() { gzip_ = false; }
 
-  uint64_t getContentLength() const { return lastContentLength_; }
+  int64_t getContentLength() const { return lastContentLength_; }
+
+  const SharedHandle<SocketRecvBuffer>& getSocketRecvBuffer() const
+  {
+    return socketRecvBuffer_;
+  }
+
+  const std::string& getAllowOrigin() const
+  {
+    return allowOrigin_;
+  }
+
+  void setAllowOrigin(const std::string& allowOrigin)
+  {
+    allowOrigin_ = allowOrigin;
+  }
+
+  const SharedHandle<SocketCore>& getSocket() const
+  {
+    return socket_;
+  }
+
+  const SharedHandle<HttpHeader>& getRequestHeader() const
+  {
+    return lastRequestHeader_;
+  }
+
+  void setSecure(bool f)
+  {
+    secure_ = f;
+  }
+
+  bool getSecure() const
+  {
+    return secure_;
+  }
+
+  bool wantRead() const;
+  bool wantWrite() const;
 };
 
 } // namespace aria2
