@@ -37,48 +37,48 @@
 
 #include "common.h"
 
-#include <string>
-
 #include <gnutls/gnutls.h>
 
+#include "TLSContext.h"
 #include "DlAbortEx.h"
 
 namespace aria2 {
 
-class Logger;
-
-class TLSContext {
-private:
-  gnutls_certificate_credentials_t certCred_;
-
-  bool good_;
-
-  bool peerVerificationEnabled_;
-
-  Logger* logger_;
+class GnuTLSContext : public TLSContext {
 public:
-  TLSContext();
+  GnuTLSContext(TLSSessionSide side);
 
-  ~TLSContext();
+  virtual ~GnuTLSContext();
 
   // private key `keyfile' must be decrypted.
-  bool addClientKeyFile(const std::string& certfile,
-                        const std::string& keyfile);
+  virtual bool addCredentialFile(const std::string& certfile,
+                                 const std::string& keyfile);
+
+  virtual bool addSystemTrustedCACerts();
 
   // certfile can contain multiple certificates.
-  bool addTrustedCACertFile(const std::string& certfile);
+  virtual bool addTrustedCACertFile(const std::string& certfile);
 
-  bool good() const;
+  virtual bool good() const;
 
-  bool bad() const;
+  virtual TLSSessionSide getSide() const {
+    return side_;
+  }
+
+  virtual bool getVerifyPeer() const {
+    return verifyPeer_;
+  }
+  virtual void setVerifyPeer(bool verify) {
+    verifyPeer_ = verify;
+  }
 
   gnutls_certificate_credentials_t getCertCred() const;
 
-  void enablePeerVerification();
-
-  void disablePeerVerification();
-
-  bool peerVerificationEnabled() const;
+private:
+  gnutls_certificate_credentials_t certCred_;
+  TLSSessionSide side_;
+  bool good_;
+  bool verifyPeer_;
 };
 
 } // namespace aria2
