@@ -36,6 +36,7 @@
 
 #include <cassert>
 #include <cstdio>
+#include <cstring>
 #include <utility>
 #include <algorithm>
 #include <numeric>
@@ -56,89 +57,42 @@
 #include "FileEntry.h"
 #include "a2io.h"
 #include "LogFactory.h"
+#include "uri.h"
+#include "SegList.h"
+#include "array_fun.h"
+#include "help_tags.h"
+#ifdef ENABLE_MESSAGE_DIGEST
+# include "MessageDigest.h"
+#endif // ENABLE_MESSAGE_DIGEST
 
 namespace aria2 {
 
-NullOptionHandler::~NullOptionHandler() {}
-
-bool NullOptionHandler::canHandle(const std::string& optName) { return true; }
-
-void NullOptionHandler::parse(Option& option, const std::string& arg) {}
-
-bool NullOptionHandler::hasTag(const std::string& tag) const { return false; }
-
-void NullOptionHandler::addTag(const std::string& tag) {}
-
-std::string NullOptionHandler::toTagString() const { return A2STR::NIL; }
-
-const std::string& NullOptionHandler::getName() const { return A2STR::NIL; }
-
-const std::string& NullOptionHandler::getDescription() const
-{
-  return A2STR::NIL;
-}
-
-const std::string& NullOptionHandler::getDefaultValue() const
-{
-  return A2STR::NIL;
-}
-
-std::string NullOptionHandler::createPossibleValuesString() const
-{
-  return A2STR::NIL;
-}
-
-bool NullOptionHandler::isHidden() const
-{
-  return true;
-}
-
-void NullOptionHandler::hide() {}
-
-OptionHandler::ARG_TYPE NullOptionHandler::getArgType() const
-{
-  return OptionHandler::NO_ARG;
-}
-
-int NullOptionHandler::getOptionID() const
-{
-  return id_;
-}
-
-void NullOptionHandler::setOptionID(int id)
-{
-  id_ = id;
-}
-
-char NullOptionHandler::getShortName() const
-{
-  return 0;
-}
-
 BooleanOptionHandler::BooleanOptionHandler
-(const std::string& optName,
- const std::string& description,
+(const Pref* pref,
+ const char* description,
  const std::string& defaultValue,
  OptionHandler::ARG_TYPE argType,
  char shortName)
-  : NameMatchOptionHandler(optName, description, defaultValue,
-                           argType, shortName)
+  : AbstractOptionHandler(pref, description, defaultValue,
+                          argType, shortName)
 {}
 
 BooleanOptionHandler::~BooleanOptionHandler() {}
 
 void BooleanOptionHandler::parseArg(Option& option, const std::string& optarg)
+  const
 {
   if(optarg == "true" ||
      ((argType_ == OptionHandler::OPT_ARG ||
        argType_ == OptionHandler::NO_ARG)
       && optarg.empty())) {
-    option.put(optName_, A2_V_TRUE);
+    option.put(pref_, A2_V_TRUE);
   } else if(optarg == "false") {
-    option.put(optName_, A2_V_FALSE);
+    option.put(pref_, A2_V_FALSE);
   } else {
-    std::string msg = optName_;
-    strappend(msg, " ", _("must be either 'true' or 'false'."));
+    std::string msg = pref_->k;
+    msg += " ";
+    msg += _("must be either 'true' or 'false'.");
     throw DL_ABORT_EX(msg);
   }
 }
@@ -149,13 +103,13 @@ std::string BooleanOptionHandler::createPossibleValuesString() const
 }
 
 IntegerRangeOptionHandler::IntegerRangeOptionHandler
-(const std::string& optName,
- const std::string& description,
+(const Pref* pref,
+ const char* description,
  const std::string& defaultValue,
  int32_t min, int32_t max,
  char shortName)
-  : NameMatchOptionHandler(optName, description, defaultValue,
-                           OptionHandler::REQ_ARG, shortName),
+  : AbstractOptionHandler(pref, description, defaultValue,
+                          OptionHandler::REQ_ARG, shortName),
     min_(min),
     max_(max)
 {}
@@ -163,36 +117,37 @@ IntegerRangeOptionHandler::IntegerRangeOptionHandler
 IntegerRangeOptionHandler::~IntegerRangeOptionHandler() {}
 
 void IntegerRangeOptionHandler::parseArg
-(Option& option, const std::string& optarg)
+(Option& option, const std::string& optarg) const
 {
-  IntSequence seq = util::parseIntRange(optarg);
-  while(seq.hasNext()) {
-    int32_t v = seq.next();
+  SegList<int> sgl;
+  util::parseIntSegments(sgl, optarg);
+  sgl.normalize();
+  while(sgl.hasNext()) {
+    int v = sgl.next();
     if(v < min_ || max_ < v) {
-      std::string msg = optName_;
-      strappend(msg, " ", _("must be between %s and %s."));
-      throw DL_ABORT_EX
-        (fmt(msg.c_str(), util::itos(min_).c_str(),
-             util::itos(max_).c_str()));
+      std::string msg = pref_->k;
+      msg += " ";
+      msg += _("must be between %d and %d.");
+      throw DL_ABORT_EX(fmt(msg.c_str(), min_, max_));
     }
-    option.put(optName_, optarg);
+    option.put(pref_, optarg);
   }
 }
 
 std::string IntegerRangeOptionHandler::createPossibleValuesString() const
 {
-  return util::itos(min_)+"-"+util::itos(max_);
+  return fmt("%d-%d", min_, max_);
 }
 
 NumberOptionHandler::NumberOptionHandler
-(const std::string& optName,
- const std::string& description,
+(const Pref* pref,
+ const char* description,
  const std::string& defaultValue,
  int64_t min,
  int64_t max,
  char shortName)
-  : NameMatchOptionHandler(optName, description, defaultValue,
-                           OptionHandler::REQ_ARG, shortName),
+  : AbstractOptionHandler(pref, description, defaultValue,
+                          OptionHandler::REQ_ARG, shortName),
     min_(min),
     max_(max)
 {}
@@ -200,28 +155,30 @@ NumberOptionHandler::NumberOptionHandler
 NumberOptionHandler::~NumberOptionHandler() {}
 
 void NumberOptionHandler::parseArg(Option& option, const std::string& optarg)
+  const
 {
-  int64_t num = util::parseLLInt(optarg);
-  parseArg(option, num);
+  int64_t number;
+  if(util::parseLLIntNoThrow(number, optarg)) {
+    parseArg(option, number);
+  } else {
+    throw DL_ABORT_EX(fmt("Bad number %s", optarg.c_str()));
+  }
 }
 
-void NumberOptionHandler::parseArg(Option& option, int64_t number)
+void NumberOptionHandler::parseArg(Option& option, int64_t number) const
 {
   if((min_ == -1 || min_ <= number) && (max_ ==  -1 || number <= max_)) {
-    option.put(optName_, util::itos(number));
+    option.put(pref_, util::itos(number));
   } else {
-    std::string msg = optName_;
+    std::string msg = pref_->k;
     msg += " ";
     if(min_ == -1 && max_ != -1) {
-      msg += fmt(_("must be smaller than or equal to %s."),
-                 util::itos(max_).c_str());
+      msg += fmt(_("must be smaller than or equal to %" PRId64 "."), max_);
     } else if(min_ != -1 && max_ != -1) {
-      msg += fmt(_("must be between %s and %s."),
-                 util::itos(min_).c_str(),
-                 util::itos(max_).c_str());
+      msg += fmt(_("must be between %" PRId64 " and %" PRId64 "."),
+                 min_, max_);
     } else if(min_ != -1 && max_ == -1) {
-      msg += fmt(_("must be greater than or equal to %s."),
-                 util::itos(min_).c_str());
+      msg += fmt(_("must be greater than or equal to %" PRId64 "."), min_);
     } else {
       msg += _("must be a number.");
     }
@@ -247,34 +204,34 @@ std::string NumberOptionHandler::createPossibleValuesString() const
 }
 
 UnitNumberOptionHandler::UnitNumberOptionHandler
-(const std::string& optName,
- const std::string& description,
+(const Pref* pref,
+ const char* description,
  const std::string& defaultValue,
  int64_t min,
  int64_t max,
  char shortName)
-  : NumberOptionHandler(optName, description, defaultValue, min, max,
+  : NumberOptionHandler(pref, description, defaultValue, min, max,
                         shortName)
 {}
 
 UnitNumberOptionHandler::~UnitNumberOptionHandler() {}
 
 void UnitNumberOptionHandler::parseArg
-(Option& option, const std::string& optarg)
+(Option& option, const std::string& optarg) const
 {
   int64_t num = util::getRealSize(optarg);
   NumberOptionHandler::parseArg(option, num);
 }
 
 FloatNumberOptionHandler::FloatNumberOptionHandler
-(const std::string& optName,
- const std::string& description,
+(const Pref* pref,
+ const char* description,
  const std::string& defaultValue,
  double min,
  double max,
  char shortName)
-  : NameMatchOptionHandler(optName, description, defaultValue,
-                           OptionHandler::REQ_ARG, shortName),
+  : AbstractOptionHandler(pref, description, defaultValue,
+                          OptionHandler::REQ_ARG, shortName),
     min_(min),
     max_(max)
 {}
@@ -282,13 +239,13 @@ FloatNumberOptionHandler::FloatNumberOptionHandler
 FloatNumberOptionHandler::~FloatNumberOptionHandler() {}
 
 void FloatNumberOptionHandler::parseArg
-(Option& option, const std::string& optarg)
+(Option& option, const std::string& optarg) const
 {
   double number = strtod(optarg.c_str(), 0);
   if((min_ < 0 || min_ <= number) && (max_ < 0 || number <= max_)) {
-    option.put(optName_, optarg);
+    option.put(pref_, optarg);
   } else {
-    std::string msg = optName_;
+    std::string msg = pref_->k;
     msg += " ";
     if(min_ < 0 && max_ >= 0) {
       msg += fmt(_("must be smaller than or equal to %.1f."), max_);
@@ -309,38 +266,35 @@ std::string FloatNumberOptionHandler::createPossibleValuesString() const
   if(min_ < 0) {
     valuesString += "*";
   } else {
-    char buf[11];
-    snprintf(buf, sizeof(buf), "%.1f", min_);
-    valuesString += buf;
+    valuesString += fmt("%.1f", min_);
   }
   valuesString += "-";
   if(max_ < 0) {
     valuesString += "*";
   } else {
-    char buf[11];
-    snprintf(buf, sizeof(buf), "%.1f", max_);
-    valuesString += buf;
+    valuesString += fmt("%.1f", max_);
   }
   return valuesString;
 }
 
 DefaultOptionHandler::DefaultOptionHandler
-(const std::string& optName,
- const std::string& description,
+(const Pref* pref,
+ const char* description,
  const std::string& defaultValue,
  const std::string& possibleValuesString,
  OptionHandler::ARG_TYPE argType,
  char shortName)
-  : NameMatchOptionHandler(optName, description, defaultValue, argType,
-                           shortName),
+  : AbstractOptionHandler(pref, description, defaultValue, argType,
+                          shortName),
     possibleValuesString_(possibleValuesString)
 {}
 
 DefaultOptionHandler::~DefaultOptionHandler() {}
 
 void DefaultOptionHandler::parseArg(Option& option, const std::string& optarg)
+  const
 {
-  option.put(optName_, optarg);
+  option.put(pref_, optarg);
 }
 
 std::string DefaultOptionHandler::createPossibleValuesString() const
@@ -349,15 +303,15 @@ std::string DefaultOptionHandler::createPossibleValuesString() const
 }
 
 CumulativeOptionHandler::CumulativeOptionHandler
-(const std::string& optName,
- const std::string& description,
+(const Pref* pref,
+ const char* description,
  const std::string& defaultValue,
  const std::string& delim,
  const std::string& possibleValuesString,
  OptionHandler::ARG_TYPE argType,
  char shortName)
-  : NameMatchOptionHandler(optName, description, defaultValue, argType,
-                           shortName),
+  : AbstractOptionHandler(pref, description, defaultValue, argType,
+                          shortName),
     delim_(delim),
     possibleValuesString_(possibleValuesString)
 {}
@@ -365,11 +319,12 @@ CumulativeOptionHandler::CumulativeOptionHandler
 CumulativeOptionHandler::~CumulativeOptionHandler() {}
 
 void CumulativeOptionHandler::parseArg
-(Option& option, const std::string& optarg)
+(Option& option, const std::string& optarg) const
 {
-  std::string value = option.get(optName_);
-  strappend(value, optarg, delim_);
-  option.put(optName_, value);
+  std::string value = option.get(pref_);
+  value += optarg;
+  value += delim_;
+  option.put(pref_, value);
 }
 
 std::string CumulativeOptionHandler::createPossibleValuesString() const
@@ -378,22 +333,24 @@ std::string CumulativeOptionHandler::createPossibleValuesString() const
 }
 
 IndexOutOptionHandler::IndexOutOptionHandler
-(const std::string& optName,
- const std::string& description,
+(const Pref* pref,
+ const char* description,
  char shortName)
-  : NameMatchOptionHandler(optName, description, NO_DEFAULT_VALUE,
-                           OptionHandler::REQ_ARG, shortName)
+  : AbstractOptionHandler(pref, description, NO_DEFAULT_VALUE,
+                          OptionHandler::REQ_ARG, shortName)
 {}
 
 IndexOutOptionHandler::~IndexOutOptionHandler() {}
 
 void IndexOutOptionHandler::parseArg(Option& option, const std::string& optarg)
+  const
 {
   // See optarg is in the fomrat of "INDEX=PATH"
   util::parseIndexPath(optarg);
-  std::string value = option.get(optName_);
-  strappend(value, optarg, "\n");
-  option.put(optName_, value);
+  std::string value = option.get(pref_);
+  value += optarg;
+  value += "\n";
+  option.put(pref_, value);
 }
 
 std::string IndexOutOptionHandler::createPossibleValuesString() const
@@ -401,80 +358,116 @@ std::string IndexOutOptionHandler::createPossibleValuesString() const
   return "INDEX=PATH";
 }
 
+#ifdef ENABLE_MESSAGE_DIGEST
+ChecksumOptionHandler::ChecksumOptionHandler
+(const Pref* pref,
+ const char* description,
+ char shortName)
+  : AbstractOptionHandler(pref, description, NO_DEFAULT_VALUE,
+                          OptionHandler::REQ_ARG, shortName)
+{}
+
+ChecksumOptionHandler::~ChecksumOptionHandler() {}
+
+void ChecksumOptionHandler::parseArg(Option& option, const std::string& optarg)
+  const
+{
+  std::pair<Scip, Scip> p;
+  util::divide(p, optarg.begin(), optarg.end(), '=');
+  std::string hashType(p.first.first, p.first.second);
+  std::string hexDigest(p.second.first, p.second.second);
+  util::lowercase(hashType);
+  util::lowercase(hexDigest);
+  if(!MessageDigest::isValidHash(hashType, hexDigest)) {
+    throw DL_ABORT_EX(_("Unrecognized checksum"));
+  }
+  option.put(pref_, optarg);
+}
+
+std::string ChecksumOptionHandler::createPossibleValuesString() const
+{
+  return "HASH_TYPE=HEX_DIGEST";
+}
+#endif // ENABLE_MESSAGE_DIGEST
+
 ParameterOptionHandler::ParameterOptionHandler
-(const std::string& optName,
- const std::string& description,
+(const Pref* pref,
+ const char* description,
  const std::string& defaultValue,
  const std::vector<std::string>& validParamValues,
  char shortName)
-  : NameMatchOptionHandler(optName, description, defaultValue,
-                           OptionHandler::REQ_ARG, shortName),
+  : AbstractOptionHandler(pref, description, defaultValue,
+                          OptionHandler::REQ_ARG, shortName),
     validParamValues_(validParamValues)
 {}
 
 ParameterOptionHandler::ParameterOptionHandler
-(const std::string& optName,
- const std::string& description,
+(const Pref* pref,
+ const char* description,
  const std::string& defaultValue,
  const std::string& validParamValue,
  char shortName)
-  : NameMatchOptionHandler(optName, description, defaultValue,
-                           OptionHandler::REQ_ARG, shortName)
+  : AbstractOptionHandler(pref, description, defaultValue,
+                          OptionHandler::REQ_ARG, shortName)
 {
   validParamValues_.push_back(validParamValue);
 }
 
 ParameterOptionHandler::ParameterOptionHandler
-(const std::string& optName,
- const std::string& description,
+(const Pref* pref,
+ const char* description,
  const std::string& defaultValue,
  const std::string& validParamValue1,
  const std::string& validParamValue2,
  char shortName)
-  : NameMatchOptionHandler(optName, description, defaultValue,
-                           OptionHandler::REQ_ARG, shortName)
+  : AbstractOptionHandler(pref, description, defaultValue,
+                          OptionHandler::REQ_ARG, shortName)
 {
   validParamValues_.push_back(validParamValue1);
   validParamValues_.push_back(validParamValue2);
 }
 
 ParameterOptionHandler::ParameterOptionHandler
-(const std::string& optName,
- const std::string& description,
+(const Pref* pref,
+ const char* description,
  const std::string& defaultValue,
  const std::string& validParamValue1,
  const std::string& validParamValue2,
  const std::string& validParamValue3,
  char shortName)
-  : NameMatchOptionHandler(optName, description, defaultValue,
-                           OptionHandler::REQ_ARG, shortName)
+  : AbstractOptionHandler(pref, description, defaultValue,
+                          OptionHandler::REQ_ARG, shortName)
 {
   validParamValues_.push_back(validParamValue1);
   validParamValues_.push_back(validParamValue2);
   validParamValues_.push_back(validParamValue3);
 }
-   
+
 ParameterOptionHandler::~ParameterOptionHandler() {}
 
 void ParameterOptionHandler::parseArg(Option& option, const std::string& optarg)
+  const
 {
   std::vector<std::string>::const_iterator itr =
     std::find(validParamValues_.begin(), validParamValues_.end(), optarg);
   if(itr == validParamValues_.end()) {
-    std::string msg = optName_;
-    strappend(msg, " ", _("must be one of the following:"));
+    std::string msg = pref_->k;
+    msg += " ";
+    msg += _("must be one of the following:");
     if(validParamValues_.size() == 0) {
       msg += "''";
     } else {
       for(std::vector<std::string>::const_iterator itr =
             validParamValues_.begin(), eoi = validParamValues_.end();
           itr != eoi; ++itr) {
-        strappend(msg, "'", *itr, "' ");
+        msg += "'";
+        msg += *itr;
+        msg += "' ";
       }
     }
     throw DL_ABORT_EX(msg);
   } else {
-    option.put(optName_, optarg);
+    option.put(pref_, optarg);
   }
 }
 
@@ -487,14 +480,14 @@ std::string ParameterOptionHandler::createPossibleValuesString() const
 }
 
 HostPortOptionHandler::HostPortOptionHandler
-(const std::string& optName,
- const std::string& description,
+(const Pref* pref,
+ const char* description,
  const std::string& defaultValue,
- const std::string& hostOptionName,
- const std::string& portOptionName,
+ const Pref* hostOptionName,
+ const Pref* portOptionName,
  char shortName)
-  : NameMatchOptionHandler(optName, description, defaultValue,
-                           OptionHandler::REQ_ARG, shortName),
+  : AbstractOptionHandler(pref, description, defaultValue,
+                          OptionHandler::REQ_ARG, shortName),
     hostOptionName_(hostOptionName),
     portOptionName_(portOptionName)
 {}
@@ -502,6 +495,7 @@ HostPortOptionHandler::HostPortOptionHandler
 HostPortOptionHandler::~HostPortOptionHandler() {}
 
 void HostPortOptionHandler::parseArg(Option& option, const std::string& optarg)
+  const
 {
   std::string uri = "http://";
   uri += optarg;
@@ -509,12 +503,12 @@ void HostPortOptionHandler::parseArg(Option& option, const std::string& optarg)
   if(!req.setUri(uri)) {
     throw DL_ABORT_EX(_("Unrecognized format"));
   }
-  option.put(optName_, optarg);
+  option.put(pref_, optarg);
   setHostAndPort(option, req.getHost(), req.getPort());
 }
 
 void HostPortOptionHandler::setHostAndPort
-(Option& option, const std::string& hostname, uint16_t port)
+(Option& option, const std::string& hostname, uint16_t port) const
 {
   option.put(hostOptionName_, hostname);
   option.put(portOptionName_, util::uitos(port));
@@ -525,109 +519,25 @@ std::string HostPortOptionHandler::createPossibleValuesString() const
   return "HOST:PORT";
 }
 
-HttpProxyUserOptionHandler::HttpProxyUserOptionHandler
-(const std::string& optName,
- const std::string& description,
- const std::string& defaultValue,
- char shortName)
-  : NameMatchOptionHandler(optName, description, defaultValue,
-                           OptionHandler::REQ_ARG, shortName)
-{}
-
-void HttpProxyUserOptionHandler::parseArg
-(Option& option, const std::string& optarg)
-{
-  if(util::endsWith(optName_, "-user")) {
-    const std::string proxyPref = optName_.substr(0, optName_.size()-5);
-    const std::string& olduri = option.get(proxyPref);
-    if(!olduri.empty()) {
-      Request req;
-      bool b = req.setUri(olduri);
-      assert(b);
-      std::string uri = "http://";
-      if(!optarg.empty()) {
-        uri += util::percentEncode(optarg);
-      }
-      if(req.hasPassword()) {
-        uri += A2STR::COLON_C;
-        uri += util::percentEncode(req.getPassword());
-      }
-      if(uri.size() > 7) {
-        uri += "@";
-      }
-      strappend(uri, req.getHost(),A2STR::COLON_C,util::uitos(req.getPort()));
-      option.put(proxyPref, uri);
-    }
-  }
-  option.put(optName_, optarg);
-}
-
-std::string HttpProxyUserOptionHandler::createPossibleValuesString() const
-{
-  return "";
-}
-
-HttpProxyPasswdOptionHandler::HttpProxyPasswdOptionHandler
-(const std::string& optName,
- const std::string& description,
- const std::string& defaultValue,
- char shortName)
-  : NameMatchOptionHandler(optName, description, defaultValue,
-                           OptionHandler::REQ_ARG, shortName)
-{}
-
-void HttpProxyPasswdOptionHandler::parseArg
-(Option& option, const std::string& optarg)
-{
-  if(util::endsWith(optName_, "-passwd")) {
-    const std::string proxyPref = optName_.substr(0, optName_.size()-7);
-    const std::string& olduri = option.get(proxyPref);
-    if(!olduri.empty()) {
-      Request req;
-      bool b = req.setUri(olduri);
-      assert(b);
-      std::string uri = "http://";
-      if(!req.getUsername().empty()) {
-        uri += util::percentEncode(req.getUsername());
-      }
-      uri += A2STR::COLON_C;
-      if(!optarg.empty()) {
-        uri += util::percentEncode(optarg);
-      }
-      if(uri.size() > 7) {
-        uri += "@";
-      }
-      strappend(uri, req.getHost(), A2STR::COLON_C,util::itos(req.getPort()));
-      option.put(proxyPref, uri);
-    }
-  }
-  option.put(optName_, optarg);
-}
-
-std::string HttpProxyPasswdOptionHandler::createPossibleValuesString() const
-{
-  return "";
-}
-
 HttpProxyOptionHandler::HttpProxyOptionHandler
-(const std::string& optName,
- const std::string& description,
+(const Pref* pref,
+ const char* description,
  const std::string& defaultValue,
  char shortName)
-  : NameMatchOptionHandler(optName, description, defaultValue,
-                           OptionHandler::REQ_ARG, shortName),
-    proxyUserPref_(optName_+"-user"),
-    proxyPasswdPref_(optName_+"-passwd")
+  : AbstractOptionHandler(pref, description, defaultValue,
+                          OptionHandler::REQ_ARG, shortName),
+    proxyUserPref_(option::k2p(std::string(pref->k)+"-user")),
+    proxyPasswdPref_(option::k2p(std::string(pref->k)+"-passwd"))
 {}
 
 HttpProxyOptionHandler::~HttpProxyOptionHandler() {}
 
 void HttpProxyOptionHandler::parseArg(Option& option, const std::string& optarg)
+  const
 {
   if(optarg.empty()) {
-    option.put(optName_, optarg);
+    option.put(pref_, optarg);
   } else {
-    Request req;
     std::string uri;
     if(util::startsWith(optarg, "http://") ||
        util::startsWith(optarg, "https://") ||
@@ -637,31 +547,12 @@ void HttpProxyOptionHandler::parseArg(Option& option, const std::string& optarg)
       uri = "http://";
       uri += optarg;
     }
-    if(!req.setUri(uri)) {
+    uri::UriStruct us;
+    if(!uri::parse(us, uri)) {
       throw DL_ABORT_EX(_("unrecognized proxy format"));
     }
-    uri = "http://";
-    if(req.getUsername().empty()) {
-      if(option.defined(proxyUserPref_)) {
-        uri += util::percentEncode(option.get(proxyUserPref_));
-      }
-    } else {
-      uri += util::percentEncode(req.getUsername());
-    }
-    if(!req.hasPassword()) {
-      if(option.defined(proxyPasswdPref_)) {
-        uri += A2STR::COLON_C;
-        uri += util::percentEncode(option.get(proxyPasswdPref_));
-      }
-    } else {
-      uri += A2STR::COLON_C;
-      uri += util::percentEncode(req.getPassword());
-    }
-    if(uri.size() > 7) {
-      uri += "@";
-    }
-    strappend(uri, req.getHost(), A2STR::COLON_C, util::uitos(req.getPort()));
-    option.put(optName_, uri);
+    us.protocol = "http";
+    option.put(pref_, uri::construct(us));
   }
 }
 
@@ -671,30 +562,30 @@ std::string HttpProxyOptionHandler::createPossibleValuesString() const
 }
 
 LocalFilePathOptionHandler::LocalFilePathOptionHandler
-(const std::string& optName,
- const std::string& description,
+(const Pref* pref,
+ const char* description,
  const std::string& defaultValue,
  bool acceptStdin,
  char shortName)
-  : NameMatchOptionHandler(optName, description, defaultValue,
-                           OptionHandler::REQ_ARG, shortName),
+  : AbstractOptionHandler(pref, description, defaultValue,
+                          OptionHandler::REQ_ARG, shortName),
     acceptStdin_(acceptStdin)
 {}
 
 void LocalFilePathOptionHandler::parseArg
-(Option& option, const std::string& optarg)
+(Option& option, const std::string& optarg) const
 {
   if(acceptStdin_ && optarg == "-") {
-    option.put(optName_, DEV_STDIN);
+    option.put(pref_, DEV_STDIN);
   } else {
     File f(optarg);
     if(!f.exists() || f.isDir()) {
       throw DL_ABORT_EX(fmt(MSG_NOT_FILE, optarg.c_str()));
     }
-    option.put(optName_, optarg);
+    option.put(pref_, optarg);
   }
 }
-  
+
 std::string LocalFilePathOptionHandler::createPossibleValuesString() const
 {
   if(acceptStdin_) {
@@ -705,23 +596,23 @@ std::string LocalFilePathOptionHandler::createPossibleValuesString() const
 }
 
 PrioritizePieceOptionHandler::PrioritizePieceOptionHandler
-(const std::string& optName,
- const std::string& description,
+(const Pref* pref,
+ const char* description,
  const std::string& defaultValue,
  char shortName)
-  : NameMatchOptionHandler(optName, description, defaultValue,
-                           OptionHandler::REQ_ARG, shortName)
+  : AbstractOptionHandler(pref, description, defaultValue,
+                          OptionHandler::REQ_ARG, shortName)
 {}
 
 void PrioritizePieceOptionHandler::parseArg
-(Option& option, const std::string& optarg)
+(Option& option, const std::string& optarg) const
 {
   // Parse optarg against empty FileEntry list to detect syntax
   // error.
   std::vector<size_t> result;
   util::parsePrioritizePieceRange
     (result, optarg, std::vector<SharedHandle<FileEntry> >(), 1024);
-  option.put(optName_, optarg);
+  option.put(pref_, optarg);
 }
 
 std::string PrioritizePieceOptionHandler::createPossibleValuesString() const
@@ -730,23 +621,31 @@ std::string PrioritizePieceOptionHandler::createPossibleValuesString() const
 }
 
 DeprecatedOptionHandler::DeprecatedOptionHandler
-(const SharedHandle<OptionHandler>& depOptHandler,
- const std::string& repOptName)
-  : depOptHandler_(depOptHandler), repOptName_(repOptName)
-{}
-
-bool DeprecatedOptionHandler::canHandle(const std::string& optName)
+(OptionHandler* depOptHandler,
+ const OptionHandler* repOptHandler)
+  : depOptHandler_(depOptHandler), repOptHandler_(repOptHandler)
 {
-  return depOptHandler_->canHandle(optName);
+  depOptHandler_->addTag(TAG_DEPRECATED);
+}
+
+DeprecatedOptionHandler::~DeprecatedOptionHandler()
+{
+  delete depOptHandler_;
+  // We don't delete repOptHandler_.
 }
 
 void DeprecatedOptionHandler::parse(Option& option, const std::string& arg)
+  const
 {
-  A2_LOG_WARN(fmt("--%s option is deprecated. Use --%s option instead.",
-                  depOptHandler_->getName().c_str(),
-                  repOptName_.c_str()));
-  depOptHandler_->parse(option, arg);
-  option.put(repOptName_, option.get(depOptHandler_->getName()));
+  if(repOptHandler_) {
+    A2_LOG_WARN(fmt(_("--%s option is deprecated. Use --%s option instead."),
+                    depOptHandler_->getName(),
+                    repOptHandler_->getName()));
+    repOptHandler_->parse(option, arg);
+  } else {
+    A2_LOG_WARN(fmt(_("--%s option is deprecated."),
+                    depOptHandler_->getName()));
+  }
 }
 
 std::string DeprecatedOptionHandler::createPossibleValuesString() const
@@ -754,12 +653,12 @@ std::string DeprecatedOptionHandler::createPossibleValuesString() const
   return depOptHandler_->createPossibleValuesString();
 }
 
-bool DeprecatedOptionHandler::hasTag(const std::string& tag) const
+bool DeprecatedOptionHandler::hasTag(uint32_t tag) const
 {
   return depOptHandler_->hasTag(tag);
 }
 
-void DeprecatedOptionHandler::addTag(const std::string& tag)
+void DeprecatedOptionHandler::addTag(uint32_t tag)
 {
   depOptHandler_->addTag(tag);
 }
@@ -769,12 +668,12 @@ std::string DeprecatedOptionHandler::toTagString() const
   return depOptHandler_->toTagString();
 }
 
-const std::string& DeprecatedOptionHandler::getName() const
+const char* DeprecatedOptionHandler::getName() const
 {
   return depOptHandler_->getName();
 }
 
-const std::string& DeprecatedOptionHandler::getDescription() const
+const char* DeprecatedOptionHandler::getDescription() const
 {
   return depOptHandler_->getDescription();
 }
@@ -794,6 +693,11 @@ void DeprecatedOptionHandler::hide()
   depOptHandler_->hide();
 }
 
+const Pref* DeprecatedOptionHandler::getPref() const
+{
+  return depOptHandler_->getPref();
+}
+
 OptionHandler::ARG_TYPE DeprecatedOptionHandler::getArgType() const
 {
   return depOptHandler_->getArgType();
@@ -804,14 +708,64 @@ char DeprecatedOptionHandler::getShortName() const
   return depOptHandler_->getShortName();
 }
 
-int DeprecatedOptionHandler::getOptionID() const
+bool DeprecatedOptionHandler::getEraseAfterParse() const
 {
-  return depOptHandler_->getOptionID();
+  return depOptHandler_->getEraseAfterParse();
 }
 
-void DeprecatedOptionHandler::setOptionID(int id)
+void DeprecatedOptionHandler::setEraseAfterParse(bool eraseAfterParse)
 {
-  depOptHandler_->setOptionID(id);
+  depOptHandler_->setEraseAfterParse(eraseAfterParse);
+}
+
+bool DeprecatedOptionHandler::getInitialOption() const
+{
+  return depOptHandler_->getInitialOption();
+}
+
+void DeprecatedOptionHandler::setInitialOption(bool f)
+{
+  depOptHandler_->setInitialOption(f);
+}
+
+bool DeprecatedOptionHandler::getChangeOption() const
+{
+  return depOptHandler_->getChangeOption();
+}
+
+void DeprecatedOptionHandler::setChangeOption(bool f)
+{
+  depOptHandler_->setChangeOption(f);
+}
+
+bool DeprecatedOptionHandler::getChangeOptionForReserved() const
+{
+  return depOptHandler_->getChangeOptionForReserved();
+}
+
+void DeprecatedOptionHandler::setChangeOptionForReserved(bool f)
+{
+  depOptHandler_->setChangeOptionForReserved(f);
+}
+
+bool DeprecatedOptionHandler::getChangeGlobalOption() const
+{
+  return depOptHandler_->getChangeGlobalOption();
+}
+
+void DeprecatedOptionHandler::setChangeGlobalOption(bool f)
+{
+  depOptHandler_->setChangeGlobalOption(f);
+}
+
+bool DeprecatedOptionHandler::getCumulative() const
+{
+  return depOptHandler_->getCumulative();
+}
+
+void DeprecatedOptionHandler::setCumulative(bool f)
+{
+  depOptHandler_->setCumulative(f);
 }
 
 } // namespace aria2
